@@ -104,7 +104,7 @@ bicomprisk <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
   
   data <- data[order(data[,id]),]
   idtab <- table(data[,id])
-  which(data[,id]%in%names(idtab==2))
+  ##which(data[,id]%in%names(idtab==2))
   data <- data[which(data[,id]%in%names(idtab==2)),]
   if (missing(num)) {
     idtab <- table(data[,id])
@@ -121,7 +121,7 @@ bicomprisk <- function(formula, data, cause=c(1,1), cens=0, causes, indiv,
     covars2 <- paste(covars,1,sep=sep)
   for (i in seq_len(length(indiv)))
   indiv2 <- c(indiv2, paste(indiv[i],1:2,sep=sep))
-  
+
   if (oldreshape==1)
   ww0 <- reshape(data[,c(timevar,causes,covars,indiv,id,num,"lse.clusters")],
          direction="wide",idvar=id,timevar=num)[,c(id,"lse.clusters.1",timevar2,causes2,indiv2,covars2)]
