@@ -17,12 +17,13 @@
 ##'     the dyzogitic twins.
 ##' @param OS Optional. Character defining the level in the zyg variable
 ##'     corresponding to the oppposite sex dyzogitic twins.
+##' @param num Optional twin number variable
 ##' @param weight Weight matrix if needed by the chosen estimator. For use
 ##'     with Inverse Probability Weights
 ##' @param biweight Function defining the bivariate weight in each cluster
 ##' @param strata Strata
 ##' @param messages Control amount of messages shown 
-##' @param control Control argument parsed on to the optimization routine
+##' @param control Control argument parsed on to the optimization routine. Starting values may be parsed as '\code{start}'.
 ##' @param type Character defining the type of analysis to be
 ##'     performed. Should be a subset of "aced" (additive genetic factors, common
 ##'     environmental factors, unique environmental factors, dominant
@@ -37,17 +38,17 @@
 ##' @param samecens Same censoring
 ##' @param allmarg Should all marginal terms be included
 ##' @param bound Development argument
-##' @param debug Development argument
 ##' @param ... Additional arguments to lower level functions
 ##' @author Klaus K. Holst
 ##' @export
 ##' @examples
 ##' \donttest{
 ##' data(twinstut)
-##' b0 <- bptwin(stutter~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",type="u")
+##' b0 <- bptwin(stutter~sex,data=twinstut,id="tvparnr",zyg="zyg",DZ="dz",OS="os",type="ae")
 ##' summary(b0)
 ##' }
-bptwin <- function(formula, data, id, zyg, DZ, OS,
+bptwin <- function(formula, data, id, zyg, DZ, OS=NULL,
+                   num=NULL,
                    weight=NULL,
                    biweight=function(x) 1/min(x),
                    strata=NULL,
@@ -63,7 +64,7 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
                    p, indiv=FALSE,
                    constrain,
                    bound=FALSE,
-                   debug=FALSE,...) {
+                   ...) {
 
 ###{{{ setup
 
@@ -103,11 +104,11 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
 
 ##################################################
 ### No strata
-    if (is.null(control$method)) {
+  if (is.null(control$method)) {
       if (!samecens & !is.null(weight)) {
         control$method <- "bhhh"
       } else {
-        if (require(ucminf)) {
+        if (suppressWarnings(suppressPackageStartupMessages(require(ucminf)))) {
           control$method <- "gradient"
         } else control$method <- "nlminb"
       }
@@ -119,157 +120,148 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
   idtab <- table(data[,id])
   if (sum(idtab>2)) stop("More than two individuals with the same id ")
   
-  if (pairsonly)
+  if (pairsonly) {
     data <- data[as.character(data[,id])%in%names(idtab)[idtab==2],]
+    idtab <- table(data[,id])
+  }
   if (is.logical(data[,yvar])) data[,yvar] <- data[,yvar]*1
   if (is.factor(data[,yvar])) data[,yvar] <- as.numeric(data[,yvar])-1  
 
-  ## Y <- suppressWarnings(model.frame(formula,data,na.action="na.pass")[,1])    
-  ## Blen <- 0
-  ## Bord <- c()
-  ## if (degree>0)
-  ##   if (is.Surv(Y) | !missing(time) | !is.null(B)) {
-  ##     if (is.Surv(Y)) {
-  ##       if (ncol(Y)==3) {
-  ##         entry <- Y[,1]
-  ##         time <- Y[,2]
-  ##         Y <- Y[,3]*1 # event
-  ##       } else {
-  ##         time <- Y[,1]
-  ##         Y <- Y[,2]*1
-  ##       }
-  ##     } else {
-  ##       if (is.character(time))
-  ##         time <- data[,time]
-  ##       if (is.character(entry))
-  ##         entry <- data[,entry]
-  ##     }
-  ##     if (is.null(B)) {
-  ##       B <- bs(time,degree=degree)
-  ##       Bord <- cbind(time,B)[order(time),,drop=FALSE]
-  ##       colnames(B) <- paste("B","_",1:ncol(B),sep="")
-  ##       Blen <- ncol(B)   
-  ##     }
-  ##   }
-
-  idtab <- table(data[,id])
-  ##  ii0 <- which(as.character(data[,id])%in%names(idtab)[idtab==2])              
-  ##  data0 <- data[ii0,]  
-  idx2 <- NULL  
-  if (!missing(DZ)) {
-    if (!missing(OS)) {
-      warning("Opposite-sex analysis not implemented yet!")
-      idx2 <- data[,zyg]==OS
-      if (length(idx2)==0) stop("No opposite-sex twins found") 
-    }
-    idx1 <- data[,zyg]==DZ
-    if (length(idx1)==0) stop("No DZ twins found")
-    idx0 <- data[,zyg]!=DZ
-    if (length(idx1)==0) stop("No MZ twins found")
-    data[,zyg] <- (data[,zyg]!=DZ)*1
-  } else {
-    if (!missing(OS))
-      idx2 <- (as.factor(data[,zyg])==levels(as.factor(data[,zyg]))[3]) # DZos
-    DZlev <- levels(as.factor(data[,zyg]))[1]
-    idx1 <- (as.factor(data[,zyg])==DZlev) # DZ
-    idx0 <- (as.factor(data[,zyg])==levels(as.factor(data[,zyg]))[2]) # MZ
-    message("Using '",DZlev,"' as DZ",sep="")
-    data[,zyg] <- (data[,zyg]!=levels(as.factor(data[,zyg]))[1])
+  idx2 <- NULL
+  if (missing(DZ)) {
+    DZ <- levels(as.factor(data[,zyg]))[1]
+    message("Using '",DZ,"' as DZ",sep="")
   }
+  OSon <- FALSE
+  if (!is.null(OS)) {
+    idx2 <- which(data[,zyg]==OS)
+    OSon <- TRUE
+    if (length(idx2)==0) {
+      warning("No OS twins found")
+      OSon <- FALSE
+    }
+  }
+  idx1 <- which(data[,zyg]==DZ) ## DZ
+  if (length(idx1)==0) stop("No DZ twins found")
+  idx0 <- which(!(data[,zyg]%in%c(DZ,OS))) ## MZ
+  if (length(idx1)==0) stop("No MZ twins found")
+  zyg2 <- rep(1,nrow(data)); zyg2[idx0] <- 0; zyg2[idx2] <- 2
+  data[,zyg] <- zyg2 ## MZ=0, DZ=1, OS=2
+
   
+  ## time <- "time"
+  ## while (time%in%names(data)) time <- paste(time,"_",sep="")
+  ## data[,time] <- unlist(lapply(idtab,seq))
   
-  time <- "time"
-  while (time%in%names(data)) time <- paste(time,"_",sep="")
-  data[,time] <- unlist(lapply(idtab,seq))
-  
-  ##  ff <- as.formula(paste("~",paste(attributes(terms(formula))$term.labels,
-  ##                                   collapse="+"),"+1",sep=""))
-  ff <- paste(as.character(formula)[3],"+",time,"+",id,"+",zyg)
-  if (!is.null(weight))
-    ff <- paste(weight,"+",ff)
+  ## ff <- paste(as.character(formula)[3],"+",time,"+",id,"+",zyg)
+  ff <- paste(as.character(formula)[3],"+",
+              paste(c(id,zyg,weight,num),collapse="+"))
   ff <- paste("~",yvar,"+",ff)
   formula0 <- as.formula(ff)
-  Data <- model.matrix(formula0,data,na.action=na.pass)
-  rnames1 <- setdiff(colnames(Data),c(yvar,time,id,weight,zyg))
-  ##  X0 <- as.matrix(Data[,rnames1])
-  nx <-length(rnames1) ##ncol(Data)-4 + !is.null(weight)
+  opt <- options(na.action="na.pass")
+  Data <- model.matrix(formula0,data)
+  options(opt)
+  ## rnames1 <- setdiff(colnames(Data),c(yvar,time,id,weight,zyg))
+  rnames1 <- setdiff(colnames(Data),c(yvar,id,weight,zyg,num))
+  nx <- length(rnames1) 
   if (nx==0) stop("Zero design not allowed")
   
-  ##  X <- cbind(model.matrix(ff,data0)
-  ##  nx <- ncol(X)
-  ##  if (nx==0) stop("Zero design not allowed")
-##################################################
-
 
   bidx0 <- seq(nx)
-  midx0 <- bidx0; midx1 <- midx0+nx  
-  dS0 <- rbind(rep(1,4),rep(1,4),rep(1,4)) ## MZ
-  dS1 <- rbind(c(1,.5,.5,1),rep(1,4),c(1,.25,.25,1)) ## DZ
-
+  midx0 <- bidx0; midx1 <- midx0+nx
+  dS0. <- rbind(rep(1,4),rep(1,4),rep(1,4)) ## MZ
+  dS1. <- rbind(c(1,.5,.5,1),rep(1,4),c(1,.25,.25,1)) ## DZ
+  dS2.  <- rbind(c(1,0,0,1),rep(1,4),c(1,0,0,1),c(0,1,1,0))
+  
   ##mytr <- function(x) x; dmytr <- function(x) 1
   ##mytr <- function(x) x^2; dmytr <- function(x) 2*x
   ##mytr <- function(z) 1/(1+exp(-z)); dmytr <- function(z) exp(-z)/(1+exp(-z))^2
+  ACDU <- sapply(c("a","c","d","e","u"),function(x) length(grep(x,tolower(type)))>0)
+  ACDU <- c(ACDU,os=OSon)
+
   mytr <- exp; dmytr <- exp; myinvtr <- log
   trname <- "exp"; invtrname <- "log"      
-  ACDU <- sapply(c("a","c","d","e","u"),function(x) length(grep(x,tolower(type)))>0)
+
+  dmytr2 <- function(z) 4*exp(2*z)/(exp(2*z)+1)^2
+  mytr2 <- tanh;  myinvtr2 <- atanh
+  trname2 <- "tanh"; invtrname2 <- "atanh"
+
+  if (OSon) {
+    ## logit <- function(p) log(p/(1-p))
+    ## tigol <- function(z) 1/(1+exp(-z))
+    ## dlogit <- function(p) 1/(p*(1-p))
+    ## dtigol <- function(z) tigol(z)^2*exp(-z)    
+    ## mytr <- function(p) c(exp(p[-length(p)]),tigol(p[length(p)]))
+    ## myinvtr <- function(z) c(log(z[-length(z)]),logit(z[length(z)]))
+    ## dmytr <- function(p) c(exp(p[-length(p)]),dtigol(p[length(p)]))
+    mytr <- function(p) c(exp(p[-length(p)]),mytr2(p[length(p)]))
+    myinvtr <- function(z) c(log(z[-length(z)]),myinvtr2(z[length(z)]))
+    dmytr <- function(p) c(exp(p[-length(p)]),dmytr2(p[length(p)]))
+  }
+
   if (ACDU["u"]) {
     ##      datanh <- function(r) 1/(1-r^2)
-    dmytr <- function(z) 4*exp(2*z)/(exp(2*z)+1)^2
-    mytr <- tanh;  myinvtr <- atanh
-    trname <- "tanh"; invtrname <- "atanh"    
+    dmytr <- dmytr2
+    mytr <- mytr2;  myinvtr <- myinvtr2
+    trname <- trname2; invtrname <- invtrname2
     dS0 <- rbind(c(0,1,1,0))
     vidx0 <- 1
     vidx1 <- 2
-    dS1 <- dS0
+    vidx2 <- 3
+    dS2 <- dS1 <- dS0
     nvar <- length(vidx0)+length(vidx1)
+    if (OSon) nvar <- nvar+length(vidx2)
   } else {
     nvar <- sum(ACDU[1:3])
-    vidx0 <- vidx1 <- 1:nvar
-    dS0 <- dS0[ACDU[1:3],,drop=FALSE]
-    dS1 <- dS1[ACDU[1:3],,drop=FALSE]
+    vidx0 <- vidx1 <- seq(nvar); vidx2 <- seq(nvar+1)
+    if (OSon) nvar <- nvar+1
+    dS0 <- dS0.[ACDU[1:3],,drop=FALSE]
+    dS1 <- dS1.[ACDU[1:3],,drop=FALSE]
+    dS2 <- dS2.[which(c(ACDU[1:3],TRUE)),,drop=FALSE]
   }  
   if (eqmean) {
-    bidx1 <- bidx0
-##    midx0 <- midx1 <- midx    
+    bidx2 <- bidx1 <- bidx0
   } else {
     bidx1 <- bidx0+nx
-##    midx0 <- 1:nx; midx1 <- midx0+nx
-    nx <- 2*nx;
+    bidx2 <- bidx1+nx    
+    if (OSon) nx <- 3*nx else nx <- 2*nx;
   }
   
-  vidx0 <- vidx0+nx; vidx1 <- vidx1+nx
+  vidx0 <- vidx0+nx; vidx1 <- vidx1+nx; vidx2 <- vidx2+nx
   vidx <- nx+seq_len(nvar)
   midx <- seq_len(nx)
   plen <- nx+nvar
 
   Am <- matrix(c(1,.5,.5,1),ncol=2)
   Dm <- matrix(c(1,.25,.25,1),ncol=2)
-  Em <- diag(2)
+  Vm <- matrix(c(1,0,0,1),ncol=2)
+  Rm <- matrix(c(0,1,1,0),ncol=2)
 
 ##################################################
 
-  Wide <- reshape(as.data.frame(Data),idvar=c(id,zyg),timevar=time,direction="wide")
+  ## system.time(Wide <- reshape(as.data.frame(Data),idvar=c(id,zyg),timevar=time,direction="wide"))
+  ##  system.time(Wide <- as.data.frame(fast.reshape(Data,id=c(id),sep=".")))
+  Wide <- as.data.frame(fast.reshape(Data,id=c(id,zyg),sep=".",idcombine=FALSE))
   yidx <- paste(yvar,1:2,sep=".")
   rmidx <- c(id,yidx,zyg)
-  ## if (!is.null(weight)) {
-  ##   W <- cbind(data[,weight])
-  ##   widx <- paste(weight,1:2,sep=".")
-  ##   WW <- as.matrix(Wide[,widx])
-  ##   
-  ## }
+    
   W0 <- W1 <- W2 <- NULL
   if (!is.null(weight)) {
     widx <- paste(weight,1:2,sep=".")
     rmidx <- c(rmidx,widx)
-    W0 <- as.matrix(Wide[Wide[,zyg]==1,widx,drop=FALSE])
-    W1 <- as.matrix(Wide[Wide[,zyg]==0,widx,drop=FALSE])
+    W0 <- as.matrix(Wide[which(Wide[,zyg]==0),widx,drop=FALSE])
+    W1 <- as.matrix(Wide[which(Wide[,zyg]==1),widx,drop=FALSE])
+    W2 <- as.matrix(Wide[which(Wide[,zyg]==2),widx,drop=FALSE])
   }
   XX <- as.matrix(Wide[,setdiff(colnames(Wide),rmidx)])
   XX[is.na(XX)] <- 0
-  Y0 <- as.matrix(Wide[Wide[,zyg]==1,yidx,drop=FALSE])
-  Y1 <- as.matrix(Wide[Wide[,zyg]==0,yidx,drop=FALSE])
-  XX0 <- XX[Wide[,zyg]==1,,drop=FALSE]
-  XX1 <- XX[Wide[,zyg]==0,,drop=FALSE]
+
+  Y0 <- as.matrix(Wide[which(Wide[,zyg]==0),yidx,drop=FALSE])
+  Y1 <- as.matrix(Wide[which(Wide[,zyg]==1),yidx,drop=FALSE])
+  Y2 <- as.matrix(Wide[which(Wide[,zyg]==2),yidx,drop=FALSE])
+  XX0 <- XX[which(Wide[,zyg]==0),,drop=FALSE]
+  XX1 <- XX[which(Wide[,zyg]==1),,drop=FALSE]
+  XX2 <- XX[which(Wide[,zyg]==2),,drop=FALSE]
   
 ##################################################
 
@@ -280,13 +272,20 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
   ##Marginals etc.
   MyData0 <- ExMarg(Y0,XX0,W0,dS0,eqmarg=TRUE,allmarg=allmarg)
   MyData1 <- ExMarg(Y1,XX1,W1,dS1,eqmarg=TRUE,allmarg=allmarg)
-  N <- cbind(sum(idx0),sum(idx1),sum(idx2)); 
-  if (missing(OS)) N <- N[,-3,drop=FALSE]
-  N <- cbind(N,
-             2*nrow(MyData0$Y0)+nrow(MyData0$Y0_marg),
-             2*nrow(MyData1$Y0)+nrow(MyData1$Y0_marg),
-             nrow(MyData0$Y0),nrow(MyData1$Y0))
+  MyData2 <- ExMarg(Y2,XX2,W2,dS2,eqmarg=TRUE,allmarg=allmarg)
 
+  N <- cbind(length(idx0),length(idx1),length(idx2)); 
+  N <- cbind(N,
+             2*nrow(MyData0$Y0)+if (!pairsonly) NROW(MyData0$Y0_marg) else 0, 
+             2*nrow(MyData1$Y0)+if (!pairsonly) NROW(MyData1$Y0_marg) else 0,
+             2*nrow(MyData2$Y0)+if (!pairsonly) NROW(MyData2$Y0_marg) else 0,
+             NROW(MyData0$Y0),NROW(MyData1$Y0),NROW(MyData2$Y0))
+
+  
+  colnames(N) <- c("Total.MZ","Total.DZ","Total.OS","Complete.MZ","Complete.DZ","Complete.OS","Complete pairs.MZ","Complete pairs.DZ","Complete pairs.OS")
+  rownames(N) <- rep("",nrow(N))
+  if (!OSon) N <- N[,-c(3,6,9),drop=FALSE]
+  
   if (samecens & !is.null(weight)) {
     MyData0$W0 <- cbind(apply(MyData0$W0,1,biweight))
     if (!is.null(MyData0$Y0_marg))
@@ -297,55 +296,78 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
     if (!is.null(MyData1$Y0_marg))
       MyData1$W0_marg <- cbind(apply(MyData1$W0_marg,1,biweight))
   }
-  rm(Y0,XX0,W0,Y1,XX1,W1)
-  ##  N <- rbind(##c("","","Complete","","Complete pairs",""),
-  ##             rep(c("MZ","DZ"),ncol(N)/2), N)
-  colnames(N) <- c("Total.MZ","Total.DZ","Complete.MZ","Complete.DZ","Complete pairs.MZ","Complete pairs.DZ")[seq(ncol(N))]
-  rownames(N) <- rep("",nrow(N))
-##  print(N,quote=FALSE)
-  
-  ## Mu <- function(p0) {
-  ##   b0 <- cbind(p0[midx0])
-  ##   b1 <- cbind(p0[midx1])
-  ##   b00 <- b0; b11 <- b1
-  ##   if (Bconstrain) {
-  ##     b00 <- trMean(b0,Blen); b11 <- trMean(b1,Blen)
-  ##   }
-  ##     mu0 <- with(MyData0, X0%*%b00)
-  ##     mu1 <- with(MyData1, X0%*%b11)
-  ##   return(list(mu0=mu0,mu1=mu1))
-  ## }
-  Sigma <- function(p0) {    
-    p0[vidx] <- mytr(p0[vidx])    
-    if (ACDU["u"]) {     
-      ##      Sigma0 <- Em+p0[plen-1]; Sigma1 <- Em+p0[plen]
-      Sigma0 <- diag(2) + p0[plen-1]*matrix(c(0,1,1,0),2,2)
-      Sigma1 <- diag(2) + p0[plen]*matrix(c(0,1,1,0),2,2)
-    } else {    
-      pv <- ACDU*1;  pv[which(ACDU[1:3])] <- p0[vidx]
-      Sigma0 <- Em*pv["e"] + pv["a"] + pv["c"] + pv["d"]
-      Sigma1 <- Em*pv["e"] + pv["a"]*Am + pv["c"] + pv["d"]*Dm
-    }
-    return(list(Sigma0=Sigma0,Sigma1=Sigma1))
+  if (samecens & !is.null(weight)) {
+    MyData2$W0 <- cbind(apply(MyData2$W0,1,biweight))
+    if (!is.null(MyData2$Y0_marg))
+      MyData2$W0_marg <- cbind(apply(MyData2$W0_marg,1,biweight))
   }
 
-  ###}}} Mean/Var function
+  rm(Y0,XX0,W0,Y1,XX1,W1,Y2,XX2,W2)
+  
+
+  Sigma <- function(p0) {
+    Sigma2 <- NULL
+    p0[vidx] <- mytr(p0[vidx])    
+    if (ACDU["u"]) {
+      pos0 <- ifelse(OSon, plen-2, plen-1)
+      Sigma0 <- diag(2) + p0[pos0]*Rm
+      Sigma1 <- diag(2) + p0[pos0+1]*Rm
+      if (OSon) Sigma2 <- diag(2) + p0[pos0+2]*Rm
+    } else {
+      ii <- ACDU; ii[4:5] <- FALSE
+      pv <- ACDU*1;  pv[ii] <- p0[vidx]
+      Sigma0 <- Vm*pv["e"] + pv["a"] + pv["c"] + pv["d"]
+      Sigma1 <- Vm*pv["e"] + pv["a"]*Am + pv["c"] + pv["d"]*Dm
+      Sigma2 <- Vm*pv["e"] + pv["c"] + (pv["a"] + pv["d"])*Vm +
+        pv["os"]*(pv["a"] + pv["d"])*Rm
+      if (OSon) {
+        dS2 <- dS2.
+        dS2[c(1,3),2:3] <- pv["os"]
+        dS2[4,2:3] <- pv["a"]+pv["d"]
+        dS2 <- dS2[which(c(ACDU[1:3],TRUE)),]
+      }
+    }
+    return(list(Sigma0=Sigma0,Sigma1=Sigma1,Sigma2=Sigma2,dS2=dS2))
+  }
+
+  ## p0 <- op$par
+  ## ff <- function(p) as.vector(Sigma(p)$Sigma2)
+  ## jacobian(ff,p0)
+  ## Sigma(p0)$dS2
+  ## dmytr(p0[vidx])
+  ## Sigma(p0)$dS2[1,]*dmytr(p0[vidx])[1]
+  ## Sigma(p0)$dS2[2,]*dmytr(p0[vidx])[2]
+  ## Sigma(p0)$dS2[3,]*dmytr(p0[vidx])[3]
+ 
+###}}} Mean/Var function
   
 ###{{{ U  
 
+
+  p0 <- rep(-1,plen); ##p0[vidx] <- 0
+  if (OSon) p0[length(p0)] <- 0.3
+  if (type=="u")
+    p0[vidx] <- 0.3
+  if (!is.null(control$start)) {
+    p0 <- control$start
+    control$start <- NULL
+  } else {
+    X <- rbind(MyData0$XX0[,midx0,drop=FALSE],MyData0$XX0[,midx1,drop=FALSE])
+    Y <- rbind(MyData0$Y0[,1,drop=FALSE],MyData0$Y0[,2,drop=FALSE])
+    g <- suppressWarnings(glm(Y~-1+X,family=binomial(probit)))
+    p0[midx] <- coef(g)
+  }
+
+
   U <- function(p,indiv=FALSE) {
     b0 <- cbind(p[bidx0])
     b1 <- cbind(p[bidx1])
-    b00 <- b0; b11 <- b1
-    ## if (Bconstrain) {
-    ##   b00 <- trMean(b0,Blen); b11 <- trMean(b1,Blen)
-    ## }
+    b2 <- cbind(p[bidx2])
+    b00 <- b0; b11 <- b1; b22 <- b2
     if (bound) p[vidx] <- min(p[vidx],20)
     S <- Sigma(p)
     lambda <- eigen(S$Sigma0)$values
     if (any(lambda<1e-12 | lambda>1e9)) stop("Variance matrix out of bounds")
-    ##mu0 <- with(MyData0, X0%*%b00)
-    ##Mu0 <- matrix(mu0,ncol=2,byrow=TRUE)
     
     Mu0 <- with(MyData0, cbind(XX0[,midx0,drop=FALSE]%*%b00,
                                XX0[,midx1,drop=FALSE]%*%b00))
@@ -353,39 +375,60 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
                              Mu0,
                              S$Sigma0,dS0,Y0,XX0,W0,!is.null(W0),samecens))
 
-    if (!is.null(MyData0$Y0_marg)) {
+    if (!is.null(MyData0$Y0_marg) &&!pairsonly) {
       mum <- with(MyData0, XX0_marg%*%b00)
+      dSmarg <- dS0[,1,drop=FALSE]
        U_marg <- with(MyData0, .Call("uniprobit",
                                    mum,XX0_marg,
-                                   S$Sigma0[1,1],t(dS0_marg),Y0_marg,
+                                   S$Sigma0[1,1],t(dSmarg),Y0_marg,
                                    W0_marg,!is.null(W0_marg),TRUE))
       U0$score <- rbind(U0$score,U_marg$score)
       U0$loglik <- c(U0$loglik,U_marg$loglik)
     }
 
-##    mu1 <- with(MyData1, X0%*%b11) 
-##    Mu1 <- matrix(mu1,ncol=2,byrow=TRUE)
     Mu1 <- with(MyData1, cbind(XX0[,midx0,drop=FALSE]%*%b11,
                                XX0[,midx1,drop=FALSE]%*%b11))
 
     U1 <- with(MyData1, .Call("biprobit0",
                              Mu1,
                              S$Sigma1,dS1,Y0,XX0,W0,!is.null(W0),samecens))
-    if (!is.null(MyData1$Y0_marg)) {
+    if (!is.null(MyData1$Y0_marg) &&!pairsonly) {
       mum <- with(MyData1, XX0_marg%*%b11)
+      dSmarg <- dS1[,1,drop=FALSE]
       U_marg <- with(MyData1, .Call("uniprobit",
                                     mum,XX0_marg,
-                                    S$Sigma1[1,1],t(dS0_marg),Y0_marg,
+                                    S$Sigma1[1,1],t(dSmarg),Y0_marg,
                                     W0_marg,!is.null(W0_marg),TRUE))
       U1$score <- rbind(U1$score,U_marg$score)
       U1$loglik <- c(U1$loglik,U_marg$loglik)
     }
 
+    U2 <- val2 <- NULL
+    if (OSon) {
+      Mu2 <- with(MyData2, cbind(XX0[,midx0,drop=FALSE]%*%b22,
+                                 XX0[,midx1,drop=FALSE]%*%b22))
+      U2 <- with(MyData2, .Call("biprobit0",
+                                Mu2,
+                                S$Sigma2,S$dS2,Y0,XX0,W0,!is.null(W0),samecens))
+      if (!is.null(MyData2$Y0_marg) &&!pairsonly) {
+        mum <- with(MyData2, XX0_marg%*%b22)
+        dSmarg <- S$dS2[,1,drop=FALSE]
+        U_marg <- with(MyData2, .Call("uniprobit",
+                                      mum,XX0_marg,
+                                      S$Sigma2[1,1],t(dSmarg),Y0_marg,
+                                      W0_marg,!is.null(W0_marg),TRUE))
+        U2$score <- rbind(U2$score,U_marg$score)
+        U2$loglik <- c(U2$loglik,U_marg$loglik)
+      }
+    }
+
+    
     if (indiv) {
       ll0 <- U0$loglik
       ll1 <- U1$loglik
       val0 <- U0$score[MyData0$id,,drop=FALSE]
       val1 <- U1$score[MyData1$id,,drop=FALSE]
+      
       N0 <- length(MyData0$id)
       idxs0 <- seq_len(N0)
       if (length(MyData0$margidx)>0) {
@@ -397,6 +440,7 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
         val0 <- rbind(val0, U0$score[-idxs0,,drop=FALSE])
         ll0 <- c(ll0,ll0[-idxs0])
       }
+      
       N1 <- length(MyData1$id)
       idxs1 <- seq_len(N1)
       if (length(MyData1$margidx)>0) {
@@ -408,48 +452,43 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
         val1 <- rbind(val1, U1$score[-idxs1,,drop=FALSE])
         ll1 <- c(ll1,ll1[-idxs1])        
       }
-      val <- matrix(0,ncol=plen,nrow=nrow(val0)+nrow(val1))
+
+      if (OSon) {
+        ll2 <- U2$loglik
+        val2 <- U2$score[MyData2$id,,drop=FALSE]
+        N2 <- length(MyData2$id)
+        idxs2 <- seq_len(N2)
+        if (length(MyData2$margidx)>0) {
+          for (i in seq_len(N2)) {
+            idx2 <- which((MyData2$idmarg)==(MyData2$id[i]))+N2
+            idxs2 <- c(idxs2,idx2)
+            val2[i,] <- val2[i,]+colSums(U2$score[idx2,,drop=FALSE])
+          }
+          val2 <- rbind(val2, U2$score[-idxs2,,drop=FALSE])
+          ll2 <- c(ll2,ll2[-idxs2])        
+        }
+      } 
+      
+      val <- matrix(0,ncol=plen,nrow=nrow(val0)+nrow(val1) + NROW(val2))
       val[seq_len(nrow(val0)),c(bidx0,vidx0)] <- val0
       val[nrow(val0)+seq_len(nrow(val1)),c(bidx1,vidx1)] <- val1
-      for (ii in vidx) {
-        val[,ii] <- val[,ii]*dmytr(p[ii])
+      if (OSon) {
+        val[nrow(val0)+nrow(val1)+seq_len(nrow(val2)),c(bidx2,vidx2)] <- val2
       }
-      attributes(val)$logLik <- c(U0$loglik,U1$loglik)
+      trp <- dmytr(p[vidx])
+      for (i in seq(length(vidx))) {
+        val[,vidx[i]] <- val[,vidx[i]]*trp[i]
+      }
+      attributes(val)$logLik <- c(U0$loglik,U1$loglik,U2$loglik)
       return(val)
       
-#########      
-      ## val <- matrix(0,ncol=plen,nrow=nrow(U0$score)+nrow(U1$score))
-      ## val[seq_len(nrow(U0$score)),c(bidx0,vidx0)] <- U0$score
-      ## val[nrow(U0$score)+seq_len(nrow(U1$score)),c(bidx1,vidx1)] <- U1$score
-      ## for (ii in vidx) {
-      ##   val[,ii] <- val[,ii]*dmytr(p[ii])
-      ## }
-      ## ## if (Bconstrain & Blen>0) {
-      ## ##   Bidx <- attributes(b00)$idx
-      ## ##   val[,Bidx] <- as.numeric((val[,Bidx,drop=FALSE])%*%attributes(b00)$D[Bidx,Bidx,drop=FALSE])
-      ## ##   if (!eqmean) {
-      ## ##     Bidx <- bidx1[attributes(b11)$idx]
-      ## ##     val[,Bidx] <- as.numeric((val[,Bidx,drop=FALSE])%*%attributes(b11)$D[attributes(b11)$idx,attributes(b11)$idx])
-      ## ##   }
-      ## ## }      
-      ## attributes(val)$logLik <- c(U0$loglik,U1$loglik)
-      ## return(val)
-      
     }
     val <- numeric(plen)
     val[c(bidx0,vidx0)] <- colSums(U0$score)
     val[c(bidx1,vidx1)] <- val[c(bidx1,vidx1)]+colSums(U1$score)
-    for (ii in vidx)
-      val[ii] <- val[ii]*dmytr(p[ii])
-    ## if (Bconstrain & Blen>0) {
-    ##   Bidx <- attributes(b00)$idx
-    ##   val[Bidx] <- as.numeric(val[Bidx]%*%attributes(b00)$D[Bidx,Bidx])
-    ##   if (!eqmean) {
-    ##     Bidx <- bidx1[attributes(b11)$idx]
-    ##     val[Bidx] <- as.numeric(val[Bidx]%*%attributes(b11)$D[attributes(b11)$idx,attributes(b11)$idx])
-    ##   }
-    ## }    
-    attributes(val)$logLik <- sum(U0$loglik)+sum(U1$loglik)
+    if (OSon) val[c(bidx2,vidx2)] <- val[c(bidx2,vidx2)]+colSums(U2$score)
+    val[vidx] <- val[vidx]*dmytr(p[vidx])
+    attributes(val)$logLik <- sum(U0$loglik)+sum(U1$loglik)+sum(U2$loglik)
     return(val)
   }
 
@@ -457,26 +496,6 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
 
 ###{{{ optim
 
-  p0 <- rep(-1,plen); ##p0[vidx] <- 0
-  if (type=="u")
-    p0[vidx] <- 0.3
-  if (!is.null(control$start)) {
-    p0 <- control$start
-    control$start <- NULL
-  } else {
-    X <- rbind(MyData0$XX0[,midx0,drop=FALSE],MyData0$XX0[,midx1,drop=FALSE])
-    Y <- rbind(MyData0$Y0[,1,drop=FALSE],MyData0$Y0[,2,drop=FALSE])
-    g <- suppressWarnings(glm(Y~-1+X,family=binomial(probit)))
-    p0[midx] <- coef(g)
-    ## if (Blen>0) {
-    ##   pB <- p0[tail(midx,Blen)]
-    ##   pB[1] <- ifelse(pB[1]<0,-2,log(pB[1]))
-    ##   if (Blen>1) {
-    ##     pB[seq_len(Blen-1)+1] <- -2
-    ##   }
-    ##   p0[tail(midx,Blen)] <- pB
-    ## }
-  }
  
   if (!missing(p)) return(U(p,indiv=indiv))
 
@@ -506,6 +525,13 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
   }
 
 
+  ## Derivatives, Sanity check 
+  ## ff <- function(p) attributes(U(p,indiv=FALSE))$logLik
+  ## pp <- c(0,-.1,.1,0.5)
+  ## grad(ff,pp)
+  ## U(pp,indiv=FALSE)
+
+  
   controlstd <- list(hessian=0)
   controlstd[names(control)] <- control
   control <- controlstd
@@ -514,7 +540,6 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
   ucminfopt <- intersect(names(control),c("trace","grtol","xtol","stepmax","maxeval","grad","gradstep","invhessian.lt"))
   optimopt <- names(control) 
 
-  if (debug) browser()
   op <- switch(tolower(control$method),
                nlminb=nlminb(p0,f0,gradient=g0,control=control[nlminbopt]),
                optim=optim(p0,fn=f0,gr=g0,control=control[ucminfopt]),
@@ -539,22 +564,9 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
                ##                              ucminf=ucminf(p0,f,control=mycontrol[ucminfopt],hessian=F),
                ##                optim=optim(p0,f,control=mycontrol[ucminfopt],...),
                  nlminb(p0,f,control=control[nlminbopt]))
-  ##  op <- nlm(f,p0,print.level=2)
-  ##  op <- spg(p0,f,control=control)
-  
 
   if (stderr) {
-    
-    ## WW <- rbind(W0,W1)
-    ## ff <- function(p) {
-    ##   UU <- U(p,indiv=TRUE)
-    ##   s <- apply(UU,2,function(x) x) ##*WW[,1,drop=FALSE])
-    ##   l <- attributes(UU)$logLik##*WW[,1]
-    ##   res <- structure(sum(l), grad=colSums(s))
-    ##   res
-    ## }
-    
-    UU <- U(op$par,indiv=TRUE)
+    UU <- U(op$par,indiv=TRUE)    
     I <- -numDeriv::jacobian(U,op$par)
     tol <- 1e-15
     V <- Inverse(I,tol)
@@ -570,25 +582,29 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
     I <- J <- V <- matrix(NA,ncol=length(op$par),nrow=length(op$par))
   }
 
-  ###}}} optim
+###}}} optim
 
 ###{{{ return
 
-  cc <- cbind(op$par,sqrt(diag(V)))
+  suppressWarnings(cc <- cbind(op$par,sqrt(diag(V))))
   cc <- cbind(cc,cc[,1]/cc[,2],2*(1-pnorm(abs(cc[,1]/cc[,2]))))
   colnames(cc) <- c("Estimate","Std.Err","Z","p-value")
   vnames1 <- NULL
   trnam <- " "
-  if (debug) browser()
+
   if (!eqmean) {
-    rnames1 <- c(paste(rnames1,"MZ",sep=trnam),paste(rnames1,"DZ",sep=trnam))
+    rnam <- rnames1
+    rnames1 <- c(paste(rnam,"MZ",sep=trnam),paste(rnam,"DZ",sep=trnam))
+    if (OSon) rnames1 <- c(rnames1,paste(rnam,"OS",sep=trnam))
+
   }
 
   if (ACDU["u"]) {
-##    rnames <- c(rnames1,paste(c("log(var(U))","log(var(U))"),c("MZ","DZ"),sep=trnam))
-    rnames <- c(rnames1,paste(c("atanh(rho)","atanh(rho)"),c("MZ","DZ"),sep=trnam))
+    groups <- c("MZ","DZ"); if (OSon) groups <- c(groups,"OS")
+    rnames <- c(rnames1,paste(c("atanh(rho)","atanh(rho)"),groups,sep=trnam))
   } else {
     rnames <- c(rnames1,c("log(var(A))","log(var(C))","log(var(D))")[ACDU[1:3]])
+    if (OSon) rnames <- c(rnames,"atanh(rho(G[OS]))")
   }
   if (!missing(constrain)) rnames <- rnames[freeidx]
   rownames(cc) <- rnames
@@ -601,13 +617,19 @@ bptwin <- function(formula, data, id, zyg, DZ, OS,
                ACDU=ACDU[-4]*1)
   
   npar[unlist(lapply(npar,length))==0] <- 0
-##  npar$var <- nrow(cc)-sum(unlist(npar))
   
-  val <- list(coef=cc,vcov=V,score=UU,logLik=attributes(UU)$logLik,opt=op, Sigma0=S$Sigma0, Sigma1=S$Sigma1, dS0=dS0, dS1=dS1, N=N, midx0=midx0, midx1=midx1, vidx0=vidx0, vidx1=vidx1, eqmean=eqmean, I=I,J=J, robustvar=robustvar,
+  val <- list(coef=cc,vcov=V,score=UU,logLik=attributes(UU)$logLik,opt=op,
+              Sigma0=S$Sigma0, Sigma1=S$Sigma1, Sigma2=S$Sigma2,
+              dS0=dS0, dS1=dS1, dS2=dS2,
+              N=N,
+              midx0=midx0, midx1=midx1,
+              vidx0=vidx0, vidx1=vidx1, vidx2=vidx2,
+              eqmean=eqmean, I=I,J=J, robustvar=robustvar,
               transform=list(tr=mytr, invtr=myinvtr, dtr=dmytr,
                 name=trname, invname=invtrname),
-              SigmaFun=Sigma, ##MuFun=Mu,
-              npar=npar
+              SigmaFun=Sigma,
+              npar=npar,
+              OS=OSon
               )
   class(val) <- c("bptwin","biprobit")
   return(val)
