@@ -23,14 +23,21 @@ NULL
 ##' @keywords data
 NULL
 
-##' multcif data set
+##' Menarche data set (sim)
+##'
+##' @name mena
+##' @docType data
+##' @keywords data
+NULL
+
+##' Multivariate Cumulative Incidence Function example data set
 ##'
 ##' @name multcif
 ##' @docType data
 ##' @keywords data
 NULL
 
-##' stutter data set
+##' Stutter data set
 ##'
 ##' @name twinstut
 ##' @docType data
@@ -44,7 +51,7 @@ NULL
 ##' @keywords data
 NULL
 
-##' Prostate data set
+##' Prostate data set (sim)
 ##'
 ##' @name prt
 ##' @docType data
@@ -62,5 +69,6 @@ NULL
 ##' @aliases plotcr npc nonparcuminc simnordic corsim.prostate
 ##' alpha2kendall alpha2spear coefmat piecewise.twostage surv.boxarea
 ##' cluster.index familycluster.index faster.reshape piecewise.data
+##' simBinPlack simBinFam simSurvFam simCox sim 
 NULL
 
