@@ -81,7 +81,7 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
   if (!(zyg%in%colnames(data))) stop("'zyg' not found in data")
   if (!(id%in%colnames(data))) stop("'id' not found in data")
   if (missing(id)) stop("Twin-pair variable not specified")
-  
+
   if (binary | is.factor(data[,yvar]) | is.character(data[,yvar]) | is.logical(data[,yvar])) {
     args <- as.list(cl)
     args[[1]] <- NULL
@@ -121,8 +121,6 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
       return(res)
     }
   }
-
-  
   
   type <- tolower(type)
   ## if ("u" %in% type) type <- c("ue")
@@ -136,10 +134,12 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
   options(opt)
   
   covars <- colnames(mm)
-  if (attr(terms(formula),"intercept")==1)
-    covars <- covars[-1]
+  hasIntercept <- FALSE
+  if (attr(terms(formula),"intercept")==1) {
+      hasIntercept <- TRUE
+      covars <- covars[-1]
+  }
   if(length(covars)<1) covars <- NULL
-
   
   zygstat <- data[,zyg]
   if(!is.factor(zygstat)) {
@@ -147,127 +147,42 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
   }
   zyglev <- levels(zygstat)
   if (length(zyglev)>2 & missing(OS)) stop("More than two zygosity levels and no opposite sex (OS) group specified")
-  
-  ## Get data on wide format and divide into two groups by zygosity
-  if (!twinnum%in%names(data)) {
-    mynum <- rep(1,nrow(data))
-    mynum[zygstat==zyglev[1]][duplicated(data[zygstat==zyglev[1],id])] <- 2
-    mynum[zygstat==zyglev[2]][duplicated(data[zygstat==zyglev[2],id])] <- 2
-    if (!missing(OS)) {
-      mynum[zygstat==zyglev[3]][duplicated(data[zygstat==zyglev[3],id])] <- 2
-    }
-    data[,twinnum] <- mynum
-  }
-  
-  cur <- cbind(data[,c(yvar,keep)],as.data.frame(cbind(as.numeric(data[,twinnum]),as.numeric(data[,id]),as.numeric(zygstat))));
-  colnames(cur) <- c(yvar,keep,twinnum,id,zyg)
-  mydata <- cbind(cur,mm)
-  
+
+  ## To wide format:
+  num <- NULL; if (twinnum%in%colnames(data)) num <- twinnum
+  data <- cbind(data[,c(yvar,keep,num,zyg,id)],mm)
+  ddd <- fast.reshape(data,id=id,varying=c(yvar,keep,covars),keep=zyg,num=num,sep=".")
+
   if (missing(DZ)) {
     warning("Using first level, `",zyglev[1],"', in status variable as indicator for 'dizygotic'", sep="")
-    DZ <- zyglev[1]
-  }
-  myDZ <- which(levels(zygstat)==DZ)
-  myOS <- NULL
-  if (!missing(OS)) myOS <- which(levels(zygstat)==OS)
-  numlev <- seq(2+!missing(OS))
-  myMZ <- setdiff(numlev,c(myDZ,myOS))
-
-
-  data1 <- mydata[mydata[,zyg]==myMZ,,drop=FALSE]
-  if (nrow(data1)==0) stop("No MZ twins found")
-  data2 <- mydata[mydata[,zyg]==myDZ,,drop=FALSE]
-  if (nrow(data2)==0) stop("No DZ twins found")
-  data3 <- NULL
+    DZ <- zyglev[1]    
+  }  
   if (!missing(OS)) {
-    data3 <- mydata[mydata[,zyg]==myOS,,drop=FALSE]
-    if (nrow(data3)==0) stop("No opposite-sex twins found")
+    wide3 <- ddd[which(ddd[,zyg]==OS),,drop=FALSE]
+    MZ <- setdiff(zyglev,c(DZ,OS))  
+  } else {
+    wide3 <- NULL  
+    MZ <- setdiff(zyglev,DZ)
   }
+  wide1 <- ddd[which(ddd[,zyg]==MZ),,drop=FALSE]
+  wide2 <- ddd[which(ddd[,zyg]==DZ),,drop=FALSE]
+  
   
-  ## Monozygotic
-  data1.1 <- data1[which(data1[,twinnum]==1),c(id,zyg,yvar,keep,covars)]; colnames(data1.1) <- c(id,zyg,paste(colnames(data1.1)[-c(1,2)],".1",sep=""))  
-  data1.2 <- data1[which(data1[,twinnum]==2),c(id,yvar,keep,covars),drop=FALSE]; colnames(data1.2) <- c(id, paste(colnames(data1.2)[-1],".2",sep=""))
-
-  ##Missing data?
-  id1 <- data1.1[,id]
-  id2 <- data1.2[,id]
-  d1.mis <- setdiff(id2,id1) # Id's not in data1.1
-  d2.mis <- setdiff(id1,id2) # Id's not in data1.2
-  if (length(d1.mis)>0) {
-    d.temp <- matrix(NA,ncol= ncol(data1.1), nrow=length(d1.mis))
-    d.temp[,1] <- d1.mis; colnames(d.temp) <- colnames(data1.1)
-    data1.1 <- rbind(data1.1,d.temp)
-  }
-  if (length(d2.mis)>0) {
-    d.temp <- matrix(NA,ncol= ncol(data1.2), nrow=length(d2.mis))
-    d.temp[,1] <- d2.mis; colnames(d.temp) <- colnames(data1.2)
-    data1.2 <- rbind(data1.2,d.temp)
-  } 
-
-
-  ## Dizygotic 
-  data2.1 <- data2[data2[,twinnum]==1,c(id,zyg,yvar,keep,covars)]; colnames(data2.1) <- c(id,zyg,paste(colnames(data2.1)[-c(1,2)],".1",sep=""))
-  data2.2 <- data2[data2[,twinnum]==2,c(id,yvar,keep,covars),drop=FALSE]; colnames(data2.2) <- c(id, paste(colnames(data2.2)[-1],".2",sep=""))
-
-  id1 <- data2.1[,id]
-  id2 <- data2.2[,id]
-  d1.mis <- setdiff(id2,id1) # Id's not in data1.1
-  d2.mis <- setdiff(id1,id2) # Id's not in data1.2
-  if (length(d1.mis)>0) {
-    d.temp <- matrix(NA,ncol= ncol(data2.1), nrow=length(d1.mis))
-    d.temp[,1] <- d1.mis; colnames(d.temp) <- colnames(data2.1)
-    data2.1 <- rbind(data2.1,d.temp)
-  }
-  if (length(d2.mis)>0) {
-    d.temp <- matrix(NA,ncol= ncol(data2.2), nrow=length(d2.mis))
-    d.temp[,1] <- d2.mis; colnames(d.temp) <- colnames(data2.2)
-    data2.2 <- rbind(data2.2,d.temp)
-  }
-
-  wide1 <- merge(x=data1.1,y=data1.2, by=id); wide1[,zyg] <- myMZ
-  wide2 <- merge(x=data2.1,y=data2.2, by=id); wide2[,zyg] <- myDZ
-  wide3 <- NULL
-
-  if (!missing(OS)) {
-    ## Opposite sex
-    data3.1 <- data3[which(data3[,twinnum]==1),c(id,zyg,yvar,keep,covars)]; colnames(data3.1) <- c(id,zyg,paste(colnames(data3.1)[-c(1,2)],".1",sep=""))
-    data3.2 <- data3[which(data3[,twinnum]==2),c(id,yvar,keep,covars),drop=FALSE]; colnames(data3.2) <- c(id, paste(colnames(data3.2)[-1],".2",sep=""))
-
-    id1 <- data3.1[,id]
-    id2 <- data3.2[,id]
-    d1.mis <- setdiff(id2,id1) # Id's not in data3.1
-    d2.mis <- setdiff(id1,id2) # Id's not in data3.2
-    if (length(d1.mis)>0) {
-      d.temp <- matrix(NA,ncol= ncol(data3.1), nrow=length(d1.mis))
-      d.temp[,1] <- d1.mis; colnames(d.temp) <- colnames(data3.1)
-      data3.1 <- rbind(data3.1,d.temp)
-    }
-    if (length(d2.mis)>0) {
-      d.temp <- matrix(NA,ncol= ncol(data3.2), nrow=length(d2.mis))
-      d.temp[,1] <- d2.mis; colnames(d.temp) <- colnames(data3.2)
-      datta3.2 <- rbind(data3.2,d.temp)
-    }
-    wide3 <- merge(x=data3.1,y=data3.2, by=id); wide3[,zyg] <- myOS
-
-  }
-
   ## ###### The SEM
   outcomes <- paste(yvar,".",1:2,sep="")
-  model1<-lvm(outcomes,silent=TRUE)
-  f1 <- as.formula(paste(outcomes[1]," ~ ."))
-  f2 <- as.formula(paste(outcomes[2]," ~ ."))
-  regression(model1,silent=TRUE) <- update(f1, . ~ f(a1,lambda[a])+f(c1,lambda[c])+f(d1,lambda[d]) + f(e1,lambda[e])) ## + f(u,sdu1))
-  regression(model1,silent=TRUE) <- update(f2, . ~ f(a1,lambda[a])+f(c1,lambda[c])+f(d1,lambda[d]) + f(e2,lambda[e]))## + f(u,sdu1))
-  latent(model1) <- ~ a1+c1+d1+e1+e2##+u
+  model1<- lvm()  
+  regression(model1,to=outcomes,from=c("a1","c1","d1"),silent=TRUE) <-
+    rep(c("lambda[a]","lambda[c]","lambda[d]"),2)
+  regression(model1,to=outcomes,from=c("e1","e2")) <- rep("lambda[e]",2)
+  latent(model1) <- c("a1","c1","d1","e1","e2")
   intercept(model1,latent(model1)) <- 0
   if (!is.null(covars))
     for (i in 1:length(covars)) {
-      lava:::regfix(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta[",i,"]",sep="")
-      lava:::regfix(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta[",i,"]",sep="")
+      regression(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta[",i,"]",sep="")
+      regression(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta[",i,"]",sep="")
     }
-  covariance(model1) <- update(f1, . ~  v(0))
-  covariance(model1) <- update(f2, . ~  v(0))
-  lava:::covfix(model1, latent(model1), var2=NULL) <- 1
+  covariance(model1,outcomes) <- 0
+  covariance(model1, latent(model1))  <- 1
   if (!type%in%c("sat","flex")) {    
     intercept(model1,outcomes) <- "mu"
   }
@@ -276,22 +191,20 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
     covariance(model1,outcomes) <- "v1"
   }
   
-  model2 <- model1
-  cancel(model2) <- update(f2, . ~ a1)
-  cancel(model2) <- update(f2, . ~ d1)
-  regression(model2,silent=TRUE) <- update(f2, . ~ f(a2,lambda[a]))
-  regression(model2,silent=TRUE) <- update(f2, . ~ f(d2,lambda[d]))
-  
-  covariance(model2) <- a1 ~ f(a2,0.5)
-  covariance(model2) <- d1 ~ f(d2,0.25)
-  latent(model2) <- ~ a2+d2
-  intercept(model2, ~ a2+d2) <- 0
-  covariance(model2) <- c(a2,d2) ~ v(1)
+  model2 <- cancel(model1,c(outcomes[2],"a1","d1"))
+  regression(model2,to=outcomes[2],from=c("a2","d2"),silent=TRUE) <-
+    c("lambda[a]","lambda[d]")  
+  covariance(model2,a1~a2)  <- 0.5
+  covariance(model2,d1~d2)  <- 0.25
+  latent(model2) <- c("a2","d2")
+  intercept(model2, c("a2","d2")) <- 0
+  covariance(model2, c("a2","d2")) <- 1
+
   model3 <- model2
-  covariance(model3) <- a1 ~ f(a2,r1)
-  suppressMessages(constrain(model3, r1 ~ ra) <- lava:::Range.lvm())
-  covariance(model3) <- d1 ~ f(d2,r2)
-  suppressMessages(constrain(model3, r2 ~ rd) <- lava:::Range.lvm())
+  covariance(model3, a1~a2) <- "r1"
+  covariance(model3, d1~d2) <- "r2"
+  constrain(model3, r1~ra) <- function(x) tanh(x)
+  constrain(model3, r2~rd) <- function(x) tanh(x)
 
   if (type=="flex") {
      intercept(model1,outcomes) <- "mu1"
@@ -333,15 +246,14 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
     sta <- ""
     if (type=="sat") sta <- "b"
        for (i in 1:length(covars)) {
-         lava:::regfix(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta1[",i,"]",sep="")         
-         lava:::regfix(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta1",sta,"[",i,"]",sep="")
-         lava:::regfix(model2, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta2[",i,"]",sep="")
-         lava:::regfix(model2, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta2",sta,"[",i,"]",sep="")
-         lava:::regfix(model3, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta3[",i,"]",sep="")
-         lava:::regfix(model3, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta3",sta,"[",i,"]",sep="")
+         regression(model1, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta1[",i,"]",sep="")         
+         regression(model1, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta1",sta,"[",i,"]",sep="")
+         regression(model2, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta2[",i,"]",sep="")
+         regression(model2, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta2",sta,"[",i,"]",sep="")
+         regression(model3, from=paste(covars[i],".1",sep=""), to=outcomes[1],silent=TRUE) <- paste("beta3[",i,"]",sep="")
+         regression(model3, from=paste(covars[i],".2",sep=""), to=outcomes[2],silent=TRUE) <- paste("beta3",sta,"[",i,"]",sep="")
        }
   }
-
   
   full <- list(MZ=model1,DZ=model2,OS=model3)
   isA <- length(grep("a",type))>0 & type!="sat"
@@ -349,26 +261,26 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
   isD <- length(grep("d",type))>0
   isE <- length(grep("e",type))>0 | type=="sat" | type=="u"
   if (!isA) {
-    kill(model1) <- ~ a1 + a2
-    kill(model2) <- ~ a1 + a2
-    kill(model3) <- ~ a1 + a2 + ra
+    kill(model1) <- c("a1")
+    kill(model2) <- c("a1","a2")
+    kill(model3) <- c("a1","a2","ra")
     constrain(model3,r1~1) <- NULL
   }
   if (!isD) {
-    kill(model1) <- ~ d1 + d2
-    kill(model2) <- ~ d1 + d2
-    kill(model3) <- ~ d1 + d2 + rd
+    kill(model1) <- c("d1")
+    kill(model2) <- c("d1","d2")
+    kill(model3) <- c("d1","d2","rd")
     constrain(model3,r2~1) <- NULL
   }
   if (!isC) {
-    kill(model1) <- ~ c1 + c2
-    kill(model2) <- ~ c1 + c2
-    kill(model3) <- ~ c1 + c2
+    kill(model1) <- c("c1")
+    kill(model2) <- c("c1")
+    kill(model3) <- c("c1")
   }
   if (!isE) {
-    kill(model1) <- ~ e1 + e2
-    kill(model2) <- ~ e1 + e2
-    kill(model3) <- ~ e1 + e2
+    kill(model1) <- c("e1","e2")
+    kill(model2) <- c("e1","e2")
+    kill(model3) <- c("e1","e2")
   }
 
   ## Full rank covariate/design matrix?
@@ -397,7 +309,7 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
     }
 
     if (!missing(OS)) {
-      dif <- wide3[,myvars[1]]-wide2[,myvars[2]]   
+      dif <- wide3[,myvars[1]]-wide3[,myvars[2]]   
       mykeep <- myvars
       if (all(na.omit(dif)==00)) {
         mykeep <- mykeep[-2]
@@ -422,20 +334,56 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
     mm <- c(mm, OS = list(model3))
     dd <- c(dd, list(wide3))
   }
-
   names(dd) <- names(mm)
-  
-  ## suppressWarnings(mg <- multigroup(mm, dd, missing=TRUE,fix=FALSE,keep=newkeep,type=2))
 
   if (is.null(estimator)) return(multigroup(mm, dd, missing=TRUE,fix=FALSE,keep=newkeep,type=2))
-
   optim <- list(method="nlminb2",refit=FALSE,gamma=1,start=rep(0.1,length(coef(mm[[1]]))*length(mm)))
-  ##                                                       with(mg,npar+npar.mean)))
+
+
+  if (is.Surv(data[,yvar])) {
+      l <- survreg(formula,mf,dist="gaussian")
+      beta <- coef(l)
+      sigma <- l$scale
+  } else {
+      l <- lm(formula,mf)
+      beta <- coef(l)
+      sigma <- summary(l)$sigma
+  }
+  start <- rep(sigma/sqrt(nchar(type)),nchar(type))
+  if (hasIntercept) {
+      start <- c(beta[1],start)
+      start <- c(start,beta[-1])
+  } else start <- c(start,beta)
+  if (type=="sat") {
+      start <- c(rep(log(sigma^2),2),0.5)
+      if (hasIntercept) {
+          start <- c(rep(beta[1],2),start)
+          beta <- beta[-1]
+      }
+      start <- c(rep(start,2),rep(beta,4))
+  }
+  if (type=="flex") {
+      start <- c(log(sigma^2),0.5)
+      if (hasIntercept) {
+          start <- c(beta[1],start)
+          beta <- beta[-1]
+      }
+      start <- c(rep(start,2),rep(beta,2))
+  }
+  if (type=="u") {
+      start <- c(log(sigma^2),0.5,0.5)
+      if (hasIntercept) {
+          start <- c(beta[1],start)
+          beta <- beta[-1]
+      }
+      start <- c(start,beta)
+  }
+  names(start) <- NULL
+  optim$start <- start
   if (length(control)>0) {
     optim[names(control)] <- control
   }
 
-
   if (is.Surv(data[,yvar])) {
     require("lava.tobit")
     if (is.null(optim$method))
@@ -444,6 +392,7 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
   } else {
     e <- estimate(mm,dd,weight=weight,estimator=estimator,fix=FALSE,control=optim,...)
   }
+
   if (!is.null(optim$refit) && optim$refit) {
     optim$method <- "NR"
     optim$start <- pars(e)
@@ -453,7 +402,8 @@ twinlm <- function(formula, data, id, zyg, DZ, OS, strata=NULL, weight=NULL, typ
       e <- estimate(mm,dd,weight=weight,estimator=estimator,fix=FALSE,control=optim,...)
     }
   }
-  res <- list(coefficients=e$opt$estimate, vcov=Inverse(information(e)), estimate=e, model=mm, full=full, call=cl, data=data, zyg=zyg, id=id, twinnum=twinnum, type=type, model.mz=model1, model.dz=model2, model.dzos=model3, data.mz=wide1, data.dz=wide2, data.os=wide3, OS=!missing(OS), constrain=constrain)
+
+  res <- list(coefficients=e$opt$estimate, vcov=Inverse(information(e)), estimate=e, model=mm, full=full, call=cl, data=data, zyg=zyg, id=id, twinnum=twinnum, type=type, model.mz=model1, model.dz=model2, model.dzos=model3, data.mz=wide1, data.dz=wide2, data.os=wide3, OS=!missing(OS), constrain=constrain, outcomes=outcomes)
   class(res) <- "twinlm"
   return(res)
 }
