@@ -12,36 +12,40 @@
 ##' Clayton-Oakes and Plackett bivariate survival distributions,
 ##' @examples
 ##' data(diabetes)
-##'
-##' data(diabetes)
+##' 
 ##' # Marginal Cox model  with treat as covariate
 ##' margph <- coxph(Surv(time,status)~treat,data=diabetes)
-##' ### Clayton-Oakes, from timereg 
+##' ### Clayton-Oakes, from timereg
 ##' fitco1<-two.stage(margph,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id)
 ##' summary(fitco1)
-##' ### Plackett model 
-##' fitp<-twostage(margph,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id,var.link=1)
+##' ### Plackett model
+##' fitp<-twostage(margph,data=diabetes,theta=0.0,Nit=40,
+##'                clusters=diabetes$id,var.link=1)
 ##' summary(fitp)
 ##' ### Clayton-Oakes
-##' fitco2<-twostage(margph,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id,var.link=1,model="clayton.oakes")
+##' fitco2<-twostage(margph,data=diabetes,theta=1.0,detail=0,Nit=40,
+##'                  clusters=diabetes$id,var.link=1,model="clayton.oakes")
 ##' summary(fitco2)
 ##' 
 ##' ### without covariates using Aalen for marginals
 ##' marg <- aalen(Surv(time,status)~+1,data=diabetes,n.sim=0,max.clust=NULL,robust=0)
-##' fitpa<-twostage(marg,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id,score.method="optimize")
+##' fitpa<-twostage(marg,data=diabetes,theta=1.0,detail=0,Nit=40,
+##'                 clusters=diabetes$id,score.method="optimize")
 ##' summary(fitpa)
 ##' 
-##' fitcoa<-twostage(marg,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id,var.link=1,model="clayton.oakes")
+##' fitcoa<-twostage(marg,data=diabetes,theta=1.0,detail=0,Nit=40,clusters=diabetes$id,
+##'                  var.link=1,model="clayton.oakes")
 ##' summary(fitcoa)
 ##' 
 ##' ### Piecewise constant cross hazards ratio modelling
 ##' ########################################################
 ##' 
 ##' d <- subset(simClaytonOakes(2000,2,0.5,0,stoptime=2,left=0),!truncated)
-##' udp <- piecewise.twostage(c(0,0.5,2),data=d,score.method="optimize",id="cluster",timevar="time",
-##'			  status="status",model="clayton.oakes",silent=0)
+##' udp <- piecewise.twostage(c(0,0.5,2),data=d,score.method="optimize",
+##'                           id="cluster",timevar="time",
+##'                           status="status",model="clayton.oakes",silent=0)
 ##' summary(udp)
-##'
+##' 
 ##' \donttest{
 ##' ### Same model using the strata option, a bit slower
 ##' ########################################################
@@ -52,22 +56,23 @@
 ##' ud4=surv.boxarea(c(0.5,0.5),c(2,2),data=d,id="cluster",timevar="time",status="status")
 ##' ud1$strata <- 1; ud2$strata <- 2; ud3$strata <- 3; ud4$strata <- 4
 ##' ud <- rbind(ud1,ud2,ud3,ud4)
-##'
-##' marg2 <- aalen(Surv(boxtime,status)~-1+factor(num):factor(strata),data=ud,n.sim=0,robust=0)
+##' 
+##' marg2 <- aalen(Surv(boxtime,status)~-1+factor(num):factor(strata),
+##'                data=ud,n.sim=0,robust=0)
 ##' tdes <- model.matrix(~-1+factor(strata),data=ud)
-##' fitp2<-twostage(marg2,data=ud,clusters=ud$cluster,score.method="fisher.scoring",model="clayton.oakes",
+##' fitp2<-twostage(marg2,data=ud,clusters=ud$cluster,
+##'                 score.method="fisher.scoring",model="clayton.oakes",
 ##'                 theta.des=tdes,step=0.5,detail=0,strata=ud$strata)
 ##' summary(fitp2)
-##'
+##' 
 ##' ### now fitting the model with symmetry, i.e. strata 2 and 3 same effect
 ##' ud$stratas <- ud$strata; ud$stratas[ud$strata==3] <- 2;
 ##' tdes2 <- model.matrix(~-1+factor(stratas),data=ud)
-##' fitp3<-twostage(marg2,data=ud,clusters=ud$cluster,score.method="fisher.scoring",model="clayton.oakes",
+##' fitp3<-twostage(marg2,data=ud,clusters=ud$cluster,
+##'                 score.method="fisher.scoring",model="clayton.oakes",
 ##'                 theta.des=tdes2,step=0.5,detail=0,strata=ud$strata)
 ##' summary(fitp3)
 ##' }
-##' ### could also symmetry in marginal models
-##'
 ##' @keywords survival
 ##' @author Thomas Scheike
 ##' @export
@@ -87,14 +92,19 @@
 ##' @param step Step size
 ##' @param notaylor Taylor expansion
 ##' @param model model
+##' @param marginal.trunc marginal left truncation probabilities
 ##' @param marginal.survival optional vector of marginal survival probabilities 
+##' @param marginal.status related to marginal survival probabilities 
 ##' @param strata strata for fitting, see example
 ##' @param se.clusters for clusters for se calculation with iid
 ##' @param max.clust max se.clusters for se calculation with iid
 ##' @param numDeriv to get numDeriv version of second derivative, otherwise uses sum of squared score 
 twostage <- function(margsurv,data=sys.parent(),score.method="nlminb",Nit=60,detail=0,clusters=NULL,
 		     silent=1,weights=NULL, control=list(),theta=NULL,theta.des=NULL,var.link=1,iid=1,
-                     step=0.5,notaylor=0,model="plackett",marginal.survival=NULL,strata=NULL,
+                     step=0.5,notaylor=0,model="plackett",
+		     marginal.trunc=NULL,
+		     marginal.survival=NULL,
+		     marginal.status=NULL,strata=NULL,
 		     se.clusters=NULL,max.clust=NULL,numDeriv=1)
 { ## {{{
 ## {{{ seting up design and variables
@@ -102,7 +112,9 @@ rate.sim <- 1; sym=1;
 if (model=="clayton.oakes") dep.model <- 1
 else if (model=="plackett") dep.model <- 2
 else stop("Model must by either clayton.oakes or plackett \n"); 
+start.time <- NULL
 
+if (!is.null(margsurv)) 
 if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
 	 formula<-attr(margsurv,"Formula");
 	 beta.fixed <- attr(margsurv,"beta.fixed")
@@ -111,7 +123,7 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
 	 id <- attr(margsurv,"id"); 
 	 mclusters <- attr(margsurv,"cluster.call")
 	 X<-ldata$X; time<-ldata$time2; Z<-ldata$Z;  status<-ldata$status;
-	 time2 <- attr(margsurv,"stop"); start <- attr(margsurv,"start")
+	 time2 <- attr(margsurv,"stop"); start.time <- attr(margsurv,"start")
 	 antpers<-nrow(X);
          if (is.null(Z)==TRUE) {npar<-TRUE; semi<-0;}  else { Z<-as.matrix(Z); npar<-FALSE; semi<-1;}
          if (npar==TRUE) {Z<-matrix(0,antpers,1); pz<-1; fixed<-0;} else {fixed<-1;pz<-ncol(Z);}
@@ -135,9 +147,9 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
 	  if (attr(Y, "type") == "right") {
 	      time2 <- Y[, "time"]; 
 	      status <- Y[, "status"]
-		start <- rep(0,antpers);
+		start.time <- rep(0,antpers);
 		} else {
-		 start <- Y[, 1]; 
+		 start.time <- Y[, 1]; 
 		 time2 <- Y[, 2];
 		 status <- Y[, 3];
 		}
@@ -147,22 +159,24 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
 	   if (is.null(clusters)) stop("must give clusters for coxph\n");
 	   X <- matrix(1,antpers,1); ### Z <- matrix(0,antpers,1); ### no use for these
 	   px <- 1; pz <- ncol(Z); 
-	   if (sum(abs(start))>0) lefttrunk <- 1 else lefttrunk <- 0
+	   if (sum(abs(start.time))>0) lefttrunk <- 1 else lefttrunk <- 0
 	   start <- rep(0,antpers);
 	   beta.fixed <- 0
 	   semi <- 1
 ###	   start.time <- 0
 } ## }}}
 
-  if (sum(abs(start))>0) lefttrunk <- 1  else lefttrunk <- 0;  
-  RR <-  rep(1,antpers); 
-  ptrunc <- rep(1,antpers);
+  if (!is.null(start.time)) {
+  if (sum(abs(start.time))>0) lefttrunk <- 1  else lefttrunk <- 0;  
+  } else lefttrunk <- 0
 
+if (!is.null(margsurv)) 
   if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen")  { ## {{{
          resi <- residualsTimereg(margsurv,data=data) 
          RR  <- resi$RR
-	 if (lefttrunk==1) ptrunc <- exp(-resi$cumhazleft); 
 	 psurvmarg <- exp(-resi$cumhaz); 
+         ptrunc <- rep(1,length(psurvmarg)); 
+	 if (lefttrunk==1) ptrunc <- exp(-resi$cumhazleft); 
   } ## }}}
   else if (class(margsurv)=="coxph") {  ## {{{
        notaylor <- 1
@@ -170,25 +184,35 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
        cumhaz <- status-residuals
        psurvmarg <- exp(-cumhaz); 
        cumhazleft <- rep(0,antpers)
+       ptrunc <- rep(1,length(psurvmarg)); 
        RR<- exp(margsurv$linear.predictors-sum(margsurv$means*coef(margsurv)))
         if ((lefttrunk==1)) { 
          baseout <- basehaz(margsurv,centered=FALSE); 
          cum <- cbind(baseout$time,baseout$hazard)
-	 cum <- Cpred(cum,start)[,2]
+	 cum <- Cpred(cum,start.time)[,2]
 	 ptrunc <- exp(-cum * RR)
 	}
   } ## }}}
-  if (is.null(weights)==TRUE) weights <- rep(1,antpers); 
 
+
+  if (!is.null(marginal.survival) ) {
+      if (!is.null(margsurv)) {  
+	  cat("must give either margsurv model or marginal.survival=probabilities,\n"); 
+          cat(" uses marginal.survival\n"); 
+      }
+	if (lefttrunk==1)  cat("Warnings specify only your own survival weights for right-censored data\n"); 
+###        if (length(marginal.survival)!=length(start.time)) stop(paste("marginal.survival must have length=",antpers,"\n"));  
+        psurvmarg <- marginal.survival
+	antpers <- length(marginal.survival)
+        RR <-  rep(1,antpers); 
+	if (!is.null(marginal.trunc)) ptrunc <- marginal.trunc else ptrunc <- rep(1,antpers);
+	if (!is.null(marginal.status)) status <- marginal.status else stop("must give censoring status\n"); 
+  } 
+
+  if (is.null(weights)==TRUE) weights <- rep(1,antpers); 
   if (is.null(strata)==TRUE) strata<- rep(1,antpers); 
   if (length(strata)!=antpers) stop("Strata must have length equal to number of data points \n"); 
 
-  if (!is.null(marginal.survival)) {
-	if (lefttrunk==1)  cat("Warnings specify only your own survival weights for right-censored data\n"); 
-        if (length(marginal.survival)!=length(start)) stop(paste("marginal.survival must have length=",antpers,"\n"));  
-        psurvmarg <- marginal.survival
-  }
-
   cluster.call <- clusters
   out.clust <- cluster.index(clusters);  
   clusters <- out.clust$clusters
@@ -215,7 +239,8 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
 	antiid <- max.clusters
   }                                                        
 
-  ratesim<-rate.sim; pxz <- px + pz;
+  ratesim<-rate.sim; 
+###  pxz <- px + pz;
   if (is.null(theta.des)==TRUE) ptheta<-1; 
   if (is.null(theta.des)==TRUE) theta.des<-matrix(1,antpers,ptheta) else
   theta.des<-as.matrix(theta.des); 
@@ -308,7 +333,6 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
     oout <- 0; 
     tryCatch(opt <- nlminb(theta,loglike,control=control),error=function(x) NA)
     if (detail==1) print(opt); 
-
     if (detail==1 && iid==1) cat("iid decomposition\n"); 
     oout <- 2
     theta <- opt$par
@@ -319,7 +343,7 @@ if (class(margsurv)=="aalen" || class(margsurv)=="cox.aalen") { ## {{{
     if (iid==1) theta.iid <- out$theta.iid
     if (numDeriv==1) {
     if (detail==1 ) cat("numDeriv hessian start\n"); 
-      oout <- 3; 
+      oout <- 3; ## returns score 
       hess <- jacobian(loglike,opt$par)
     if (detail==1 ) cat("numDeriv hessian done\n"); 
     }
@@ -671,7 +695,6 @@ attr(ud, "Type") <- model
 return(ud);
 } ## }}}
 
-
 ##' @export
 piecewise.data <- function(cut1,cut2,data=sys.parent(),timevar="time",status="status",id="id",covars=NULL,covars.pairs=NULL,num=NULL,silent=1)
 { ## {{{
@@ -703,7 +726,6 @@ dataud <- rbind(dataud,datalr)
 return(data.frame(dataud))
 } ## }}}
 
-
 ##' @S3method summary pc.twostage
 summary.pc.twostage <- function(object,var.link=NULL,...)
 { ## {{{
@@ -758,3 +780,200 @@ coefmat <- function(est,stderr,digits=3,...) { ## {{{
   noquote(res)
 } ## }}}
 
+##' Fits two-stage model for describing depdendence in survival data
+##' using marginals that are on cox or aalen form using the twostage funcion, but
+##' call is different and easier and the data manipulation  build into the function.
+##' Useful in particular for family design data. 
+##'
+##' If clusters contain more than two times, the algoritm uses a composite likelihood
+##' based on the pairwise bivariate models.
+##'
+##' The reported standard errors are based on the estimated information from the 
+##' likelihood assuming that the marginals are known. 
+##'
+##' @examples
+##' data(prt)
+##' margp<- coxph(Surv(time,status==1)~factor(country),data=prt)
+##' fitco<-twostage(margp,data=prt,clusters=prt$id)
+##' summary(fitco)
+##' 
+##' des <- model.matrix(~-1+factor(zyg),data=prt); 
+##' fitco<-twostage(margp,data=prt,theta.des=des,clusters=prt$id)
+##' summary(fitco)
+##' 
+##' dfam <- simSurvFam(1000)
+##' dfam <- fast.reshape(dfam,var=c("x","time","status"))
+##' 
+##' desfs <- function(x,num1="num1",num2="num2")
+##' { 
+##' pp <- (x[num1]=="m")*(x[num2]=="f")*1   ## mother-father 
+##' pc <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1 ## mother-child
+##' cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1               ## child-child
+##' c(pp,pc,cc)
+##' } 
+##' 
+##' marg <- coxph(Surv(time,status)~factor(num),data=dfam)
+##' out3 <- easy.twostage(marg,data=dfam,time="time",status="status",id="id",deshelp=0,
+##'                       score.method="fisher.scoring",theta.formula=desfs,
+##'                       desnames=c("parent-parent","parent-child","child-cild"))
+##' 
+##' #summary(out3)
+##' @keywords survival twostage 
+##' @export
+##' @param margsurv model 
+##' @param data data frame
+##' @param score.method Scoring method
+##' @param status Status at exit time
+##' @param time Exit time
+##' @param entry Entry time
+##' @param id name of cluster variable in data frame
+##' @param Nit Number of iterations
+##' @param detail Detail for more output for iterations 
+##' @param silent Debug information
+##' @param weights Weights for log-likelihood, can be used for each type of outcome in 2x2 tables. 
+##' @param control Optimization arguments
+##' @param theta Starting values for variance components
+##' @param theta.formula design for depedence, either formula or design function
+##' @param desnames names for dependence parameters
+##' @param deshelp if 1 then prints out some data sets that are used, on on which the design function operates
+##' @param var.link Link function for variance (exp link)
+##' @param iid Calculate i.i.d. decomposition
+##' @param step Step size for newton-raphson
+##' @param model plackett or clayton-oakes model
+##' @param marginal.surv vector of marginal survival probabilities 
+##' @param strata strata for fitting 
+##' @param max.clust max clusters
+##' @param se.clusters clusters for iid decomposition for roubst standard errors
+easy.twostage <- function(margsurv=NULL,data=sys.parent(),score.method="nlminb",
+status="status",time="time",entry=NULL,id="id", Nit=60,detail=0, silent=1,weights=NULL, control=list(),
+theta=NULL,theta.formula=NULL,desnames=NULL,deshelp=0,var.link=1,iid=1,
+step=0.5,model="plackett",marginal.surv=NULL,strata=NULL,max.clust=NULL,se.clusters=NULL)
+{ ## {{{
+### marginal trunction probabilty, to be computed from model 
+pentry <- NULL
+
+if (is.null(marginal.surv))
+if (class(margsurv)[1]=="coxph")
+{ ## {{{
+###    ps <- survfit(margsurv)$surv
+    coxformula <- margsurv$formula
+    X <- model.matrix(coxformula,data=data)[,-1]; 
+    baseout <- basehaz(margsurv,centered=FALSE); 
+    baseout <- cbind(baseout$time,baseout$hazard)
+    cumh <-  Cpred(baseout,data[,time])[,2]
+    RR<-exp(X %*% coef(margsurv))
+    ps<-exp(-cumh*RR)
+    ## }}}
+  } else if (class(margsurv)[1]=="phreg")
+  {  ## {{{
+	  ps <- predict(margsurv)
+	  pentry <- predict(margsurv,pentry)
+  }
+  else stop("marginal survival probabilities must be given as marginal.sur or margsurv \n"); 
+
+  data <- cbind(data,ps)
+  if (!is.null(pentry)) data <- cbind(data,pentry)
+
+  ### make all pairs in the families,
+  fam <- familycluster.index(data[,id])
+  data.fam <- data[fam$familypairindex,]
+  data.fam$subfam <- fam$subfamilyindex
+
+  ### make dependency design using wide format for all pairs 
+  data.fam.clust <- fast.reshape(data.fam,id="subfam")
+  if (is.function(theta.formula)) {
+     library(compiler) 
+     desfunction <- cmpfun(theta.formula)
+    if (deshelp==1){
+ 	  cat("These names appear in wide version of pairs for dependence \n")
+	  cat("design function must be defined in terms of these: \n")
+	  cat(names(data.fam.clust)); cat("\n")
+	  cat("Here is head of wide version with pairs\n")
+	  print(head(data.fam.clust)); cat("\n")
+    }
+    des.theta  <- t( apply(data.fam.clust,1,desfunction)) 
+    colnames(des.theta) <- desnames
+    desnames <- desnames
+     } else {
+	  if (is.null(theta.formula)) theta.formula <- ~+1
+          des.theta <- model.matrix(theta.formula,data=data.fam.clust)
+          desnames <- colnames(des.theta); 
+     }
+     data.fam.clust <- cbind(data.fam.clust,des.theta)
+     if (deshelp==1) {
+	 cat("These names appear in wide version of pairs for dependence \n")
+	     print(head(data.fam.clust))
+     }
+
+    ### back to long format keeping only needed variables
+     if (is.null(pentry))
+    data.fam <- fast.reshape(data.fam.clust,varying=c(id,"ps",status))
+    else data.fam <- fast.reshape(data.fam.clust,varying=c(id,"ps",status,"pentry"))
+    if (deshelp==1) {
+	cat("Back to long format for twostage (head)\n"); 
+        print(head(data.fam)); 
+	cat("\n")
+###	cat(paste("twostage, called with reponse",response,"\n")); 
+	cat(paste("cluster=",id,",  subcluster (pairs)=subfam \n")); 
+	cat(paste("design variables =")); 
+	cat(desnames)
+	cat("\n")
+    } 
+
+###    print(status)
+###    print(names(data.fam))
+###    print(data.fam[,status])
+    if (is.null(pentry)) ptrunc <- NULL else ptrunc <- data.fam[,pentry]
+
+    out <- twostage(NULL,data=data.fam,
+                    clusters=data.fam$subfam,
+		    theta.des=data.fam[,desnames],
+                    detail=detail, score.method=score.method, Nit=Nit,step=step,
+                    iid=iid,theta=theta, var.link=var.link,model=model, 
+                    max.clust=max.clust,
+                    marginal.survival=data.fam[,"ps"],
+                    marginal.status=data.fam[,status],
+		    marginal.trunc=ptrunc,
+		    se.clusters=data.fam[,id])
+
+   return(out)
+} ## }}}
+
+### library(mets)
+### dfam <- simSurvFam(10000)
+### dfam <- fast.reshape(dfam,var=c("x","time","status"))
+###### 
+### desfs <- function(x,num1="num1",num2="num2")
+### { 
+### pp <- (x[num1]=="m")*(x[num2]=="f")*1   ## mother-father 
+### pc <- (x[num1]=="m" | x[num1]=="f")*(x[num2]=="b1" | x[num2]=="b2")*1 ## mother-child
+### cc <- (x[num1]=="b1")*(x[num2]=="b1" | x[num2]=="b2")*1               ## child-child
+### c(pp,pc,cc)
+### } 
+######
+### marg <- coxph(Surv(time,status)~factor(num),data=dfam)
+### system.time(
+###out3 <- easy.twostage(marg,data=dfam,time="time",status="status",id="id",deshelp=1,
+###  score.method="fisher.scoring",theta.formula=desfs,
+###  desnames=c("parent-parent","parent-child","child-cild"))
+###)
+
+##' @export
+simSurvFam <- function(n,beta=0.0,theta=1,lam0=0.5,lam1=1,lam2=1,ctime=10,...) { ## {{{ 
+###	n=10; beta=0; theta=1; lam1=1;lam2=1; ctime=10; lam0=0.5
+xm <- rbinom(n,1,0.5); xf <- rbinom(n,1,0.5); 
+xb1 <- rbinom(n,1,0.5); xb2 <- rbinom(n,1,0.5); 
+###
+zf <- rgamma(n,shape=lam1); zb <- rgamma(n,shape=lam2); 
+tm <- rexp(n)/(zf*exp(xm*beta)*lam0)
+tf <- rexp(n)/(zf*exp(xf*beta)*lam0)
+tb1 <- rexp(n)/((zf+zb)*exp(xb1*beta)*2*lam0)
+tb2 <- rexp(n)/((zf+zb)*exp(xb2*beta)*2*lam0)
+cm <- ifelse(tm<ctime,1,0); cf <- ifelse(tf<ctime,1,0); 
+cb1 <- ifelse(tb1<ctime,1,0); cb2 <- ifelse(tb2<ctime,1,0); 
+tm <- ifelse(tm<ctime,tm,ctime); tf <- ifelse(tf<ctime,tf,ctime)
+tb1 <- ifelse(tb1<ctime,tb1,ctime); tb2 <- ifelse(tb2<ctime,tb2,ctime)
+#
+data.frame(xm=xm,xf=xf,xb1=xb1,xb2=xb2,timem=tm,timef=tf,timeb1=tb1,timeb2=tb2,statusm=cm,statusf=cf,
+	   statusb1=cb1,statusb2=cb2,id=1:n)
+} ## }}} 
