@@ -10,6 +10,158 @@ int _mvt_inform;
 double _mvt_error[3]; 
 
 
+double dmvn(const vec &y, const mat &W, 
+	    bool log=true, double logdet=datum::inf) {
+  int n = W.n_rows;  
+  double res = -0.5*n*log2pi;
+  if (logdet!=datum::inf) {
+    res += -0.5*(logdet + as_scalar(trans(y)*W*y));    
+  } else {
+    double sign=0;
+    mat iW = inv(W);
+    log_det(logdet,sign,W);
+    res += -0.5*(logdet + as_scalar(trans(y)*iW*y));  
+  }
+  if (!log) res = exp(res);
+  return(res);  
+}
+	    
+
+double cdfmvn(mat &upper, mat &cor) {  
+  double val=0;
+  int n = cor.n_cols;
+  NumericVector _mvt_delta(n);   
+  unsigned ncor = n*(n-1)/2;
+  rowvec Cor(ncor);
+  int j = 0;
+  for (int r=0; r<n; r++) {
+    for (int c=r+1; c<n; c++) {
+      Cor(j) = cor(r,c); 
+      j++;
+    }
+  }
+  irowvec infin(n); infin.fill(0); //
+  mvtdst_(&n, &_mvt_df,
+  	  &upper[0], // Lower, ignored
+  	  &upper[0], 
+  	  &infin[0], // Infinity argument (all 0 since CDF)
+  	  &Cor[0],
+  	  &_mvt_delta[0], &_mvt_maxpts,
+  	  &_mvt_abseps, &_mvt_releps,
+  	  &_mvt_error[0], &val, &_mvt_inform);  
+  return(val);
+}
+
+
+
+RcppExport SEXP Dpmvn (SEXP lower, SEXP upper, SEXP mu, SEXP sigma, SEXP std) { 
+BEGIN_RCPP
+  colvec y = Rcpp::as<colvec>(upper);
+  NumericVector Lower(lower);
+  bool Std = Rcpp::as<bool>(std);
+  mat S = Rcpp::as<mat>(sigma);  
+  colvec Mu = Rcpp::as<colvec>(mu);  
+  int n = S.n_cols;
+  vec L(n);
+  for (int j=0; j<n; j++) {    
+    if (y(j)==datum::inf) L(j) = 1; // (lower,+Inf)
+    if (Lower(j)==-datum::inf) L(j) = 0; // (-Inf,upper)
+    // =2 (low,up)   For now we actually don't consider the interval-censored case
+    // <0 (-Inf,Inf)
+  }  
+  mat LL = diagmat(L);
+  mat iLambda;
+  if (!Std) {
+    iLambda = diagmat(1/sqrt(diagvec(S)));
+    S = iLambda*S*iLambda;    
+    y = iLambda*(y-Mu);
+  }
+  
+  List res;
+
+  vec D(n);  
+  mat H(n,n);
+  if (n==1) {
+    List res;    
+    D[0] = Rf_dnorm4(y[0],0.0,1.0,0);
+    H[0] = -y[0]*D[0];
+    if (!Std) {
+      D[0] *= iLambda[0];
+      H[0] *= iLambda[0]*iLambda[0];
+    }
+    res["gradient"] = D;
+    res["hessian"] = H;
+    return(res);    
+  }
+
+  for (unsigned j=0; j<n; j++) {
+    mat Sj = S; Sj.shed_row(j); 
+    mat Sj0 = Sj.col(j);
+    Sj.shed_col(j);
+    Sj -= Sj0*Sj0.t();
+    mat iL = diagmat(1/sqrt(diagvec(Sj)));    
+    Sj = iL*Sj*iL;
+    vec muj = y;  
+    muj.shed_row(j);
+    muj = iL*(muj - Sj0*y[j]);
+    D[j] = Rf_dnorm4(y[j],0.0,1.0,0)*
+      cdfmvn(muj,Sj);
+  }
+  
+  if (n==2) {
+    H(0,1) = H(1,0) = dmvn(y, S, false);
+    H.diag() = -y%D - H(0,1)*S(0,1);
+  } else {
+    mat phis(n,n); phis.fill(0);
+    mat Phis = phis;
+    uvec idx1(n-2);
+    uvec idx2(2);
+    for (unsigned i=0; i<(n-1); i++) {
+      for (unsigned j=(i+1); j<n; j++) {
+	idx2(0) = i; idx2(1) = j;
+	unsigned pos = 0;
+	for (unsigned k=0; k<n; k++) {
+	  if (k!=i && k!=j) {
+	    idx1(pos) = k; 
+	    pos++;
+	  }
+	}
+	mat Snij = S.submat(idx1,idx2);
+	mat S2 = S.submat(idx2,idx2);
+	mat B = Snij*inv(S2);
+	mat Sij = S.submat(idx1,idx1) - B*trans(Snij);
+	vec muij = y.elem(idx1) - B*y.elem(idx2);
+
+	mat iL = diagmat(1/sqrt(diagvec(Sij)));    
+	Sij = iL*Sij*iL;
+	muij = iL*muij;
+	Phis(i,j) = Phis(j,i) = cdfmvn(muij,Sij);
+	phis(i,j) = phis(j,i) = dmvn(y.elem(idx2),S2,false);
+      }
+    }
+    H = Phis%phis;    
+    colvec ones(n); ones.fill(1);
+    H.diag() = -y%D - (S%H)*ones;
+  }
+
+  //double val = cdfmvn(y,S);  
+  //return(Rcpp::wrap(val));
+
+  if (!Std) {
+    H = iLambda*H*iLambda;    
+    D = iLambda*D;
+  }
+  res["gradient"] = D;
+  res["hessian"] = H;
+
+  return(res);
+END_RCPP
+}
+
+
+
+
+
 vec loglikmvn(mat &Yl, mat &Yu, uvec &Status,
 	      mat &Mu, mat &S, 
 	      mat &Z,  mat &Su,
@@ -268,9 +420,8 @@ BEGIN_RCPP
       mat Musub = Mu.rows(idx);
       if (!Rf_isNull(z)) Zsub = Z.rows(idx);
       // cerr << "i=" << i << endl;
-      // cerr << "idx =" << idx << endl;
-      
-      //       cerr << "Yl =" << Ylsub << endl;
+      // cerr << "idx =" << idx << endl; 
+      // cerr << "Yl =" << Ylsub << endl;
       // cerr << "Yu =" << Yusub << endl;
       // cerr << "Musub =" << Musub << endl;
       // cerr << "Zsub =" << Zzub << endl;
